@@ -20,35 +20,26 @@ use blas_crate::{ddot, dgemm};
 use lapack_crate::{dgesv, dgetrf, dgetri};
 
 #[cfg(not(feature = "lapack"))]
-use crate::linalg::decomposition::*;
+use super::decomposition::*;
+use super::Vector;
 use crate::prelude::max;
 
 /// Generates evenly spaced values within a given interval. Values generated in the half-open
 /// interval [start, stop). That is, the stop point is not included.
-pub fn arange(start: f64, stop: f64, step: f64) -> Vec<f64> {
-    let n = (stop - start) / step + 1.;
+pub fn arange(start: f64, stop: f64, step: f64) -> Vector {
+    let n = (stop - start) / step;
     (0..n as usize)
         .map(|i| start as f64 + i as f64 * step)
-        .collect::<Vec<_>>()
+        .collect::<Vector>()
 }
 
 /// Generates evenly spaced values within a given interval, with a set number of points. Both the
 /// start and stop points are included.
-pub fn linspace(start: f64, stop: f64, num: usize) -> Vec<f64> {
+pub fn linspace(start: f64, stop: f64, num: usize) -> Vector {
     let width = (stop - start) / (num - 1) as f64;
     (0..num)
         .map(|i| start + i as f64 * width)
-        .collect::<Vec<_>>()
-}
-
-/// Checks whether a 1D array is a valid square matrix.
-pub fn is_square(m: &[f64]) -> Result<usize, String> {
-    let n = (m.len() as f32).sqrt();
-    if n % 1. == 0. {
-        Ok(n as usize)
-    } else {
-        Err("Matrix not square".to_string())
-    }
+        .collect::<Vector>()
 }
 
 /// Checks whether a 1D array is a valid matrix representation given the number of rows.
@@ -61,12 +52,22 @@ pub fn is_matrix(m: &[f64], nrows: usize) -> Result<usize, String> {
     }
 }
 
+/// Checks whether a 1D array is a valid square matrix.
+pub fn is_square(m: &[f64]) -> Result<usize, String> {
+    let n = (m.len() as f32).sqrt();
+    if n % 1. == 0. {
+        Ok(n as usize)
+    } else {
+        Err("Matrix not square".to_string())
+    }
+}
+
 /// Checks whether a 1D array is a valid design matrix, given its number of rows.
 pub fn is_design(m: &[f64], nrows: usize) -> bool {
     let mut is_design = true;
     let ncols = is_matrix(m, nrows).unwrap();
     for i in 0..nrows {
-        if m[i * ncols] != 1. {
+        if (m[i * ncols] - 1.).abs() > f64::EPSILON {
             is_design = false;
         }
     }
@@ -78,7 +79,7 @@ pub fn is_symmetric(m: &[f64]) -> bool {
     let n = is_square(m).unwrap();
     for i in 0..n {
         for j in i..n {
-            if m[i * n + j] != m[j * n + i] {
+            if (m[i * n + j] - m[j * n + i]).abs() > f64::EPSILON {
                 return false;
             }
         }
@@ -100,10 +101,19 @@ pub fn is_positive_definite(m: &[f64]) -> bool {
     true
 }
 
+pub fn diag(a: &[f64]) -> Vector {
+    let n = is_square(a).unwrap();
+    let mut results = Vector::new(Vec::with_capacity(n));
+    for i in 0..n {
+        results.push(a[i * n + i]);
+    }
+    results
+}
+
 /// Convert a 1D matrix from row-major ordering into column-major ordering.
-pub fn row_to_col_major(a: &[f64], nrows: usize) -> Vec<f64> {
+pub fn row_to_col_major(a: &[f64], nrows: usize) -> Vector {
     let ncols = is_matrix(a, nrows).unwrap();
-    let mut x = a.to_vec();
+    let mut x = Vector::new(a.to_vec());
     for i in 0..nrows {
         for j in 0..ncols {
             x[j * nrows + i] = a[i * ncols + j];
@@ -128,7 +138,7 @@ pub fn col_to_row_major(a: &[f64], nrows: usize) -> Vec<f64> {
 pub fn transpose(a: &[f64], nrows: usize) -> Vec<f64> {
     let ncols = is_matrix(&a, nrows).unwrap();
 
-    let mut at: Vec<f64> = Vec::with_capacity(a.len());
+    let mut at = Vec::with_capacity(a.len());
 
     for j in 0..ncols {
         for i in 0..nrows {
@@ -139,20 +149,10 @@ pub fn transpose(a: &[f64], nrows: usize) -> Vec<f64> {
     at
 }
 
-/// Extract the diagonal elements of a matrix.
-pub fn diag(a: &[f64]) -> Vec<f64> {
-    let n = is_square(a).unwrap();
-    let mut results = Vec::with_capacity(n);
-    for i in 0..n {
-        results.push(a[i * n + i]);
-    }
-    results
-}
-
 /// Create a diagonal matrix with the given elements along the elements.
-pub fn diag_matrix(a: &[f64]) -> Vec<f64> {
+pub fn diag_matrix(a: &[f64]) -> Vector {
     let n = a.len();
-    let mut new = vec![0.; n * n];
+    let mut new = Vector::new(vec![0.; n * n]);
     for i in 0..n {
         new[i * n + i] = a[i];
     }
@@ -205,42 +205,42 @@ pub fn solve_sys(a: &[f64], b: &[f64]) -> Vec<f64> {
 
     #[cfg(feature = "lapack")]
     {
-        let mut A = row_to_col_major(&a, n);
-        let mut B = row_to_col_major(&b, n);
+        let mut a = row_to_col_major(&a, n);
+        let mut b = row_to_col_major(&b, n);
         let mut ipiv = vec![0; n as usize];
         let mut info = 0;
         unsafe {
             dgesv(
                 n as i32,
                 nsys as i32,
-                &mut A,
+                &mut a,
                 n as i32,
                 &mut ipiv,
-                &mut B,
+                &mut b,
                 n as i32,
                 &mut info,
             );
             assert_eq!(info, 0, "dgesv failed");
         }
-        col_to_row_major(&B, n)
+        col_to_row_major(&b, n)
     }
 
     #[cfg(not(feature = "lapack"))]
     {
         let mut solutions = Vec::with_capacity(b.len());
-        let B = row_to_col_major(&b, n);
+        let b = row_to_col_major(&b, n);
 
         if is_positive_definite(a) {
             let l = cholesky(a);
             for i in 0..nsys {
-                let sol = cholesky_solve(&l, &B[(i * n)..((i + 1) * n)]);
+                let sol = cholesky_solve(&l, &b[(i * n)..((i + 1) * n)]);
                 assert_eq!(sol.len(), n);
                 solutions.extend_from_slice(&sol);
             }
         } else {
             let (lu, piv) = lu(a);
             for i in 0..nsys {
-                let sol = lu_solve(&lu, &piv, &B[(i * n)..((i + 1) * n)]);
+                let sol = lu_solve(&lu, &piv, &b[(i * n)..((i + 1) * n)]);
                 assert_eq!(sol.len(), n);
                 solutions.extend_from_slice(&sol);
             }
@@ -470,6 +470,50 @@ pub fn toeplitz(x: &[f64]) -> Vec<f64> {
     v
 }
 
+/// Calculates the sum of a vector.
+pub fn sum(x: &[f64]) -> f64 {
+    let n = x.len();
+
+    #[cfg(feature = "blas")]
+    {
+        let y = [1.];
+        unsafe { ddot(n as i32, x, 1, &y, 0) }
+    }
+
+    #[cfg(not(feature = "blas"))]
+    {
+        let chunks = (n - (n % 8)) / 8;
+        let mut s = 0.;
+
+        // unroll
+        for i in 0..chunks {
+            let idx = i * 8;
+            assert!(n > idx + 7);
+            s += x[idx]
+                + x[idx + 1]
+                + x[idx + 2]
+                + x[idx + 3]
+                + x[idx + 4]
+                + x[idx + 5]
+                + x[idx + 6]
+                + x[idx + 7];
+        }
+
+        // do the rest
+
+        // for j in (chunks * 8)..n {
+        //     s += x[j]
+        // }
+
+        // clippy says to do this instead...
+        for j in x.iter().take(n).skip(chunks * 8) {
+            s += j;
+        }
+
+        s
+    }
+}
+
 /// Calculates the dot product of two equal-length vectors. When the feature "blas" is enabled,
 /// uses `ddot` from BLAS. Otherwise, uses a length-8 unrolled loop.
 pub fn dot(x: &[f64], y: &[f64]) -> f64 {
